package digi.recipeManager;

import java.util.List;

import org.bukkit.ChatColor;
import org.bukkit.command.*;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;

import digi.recipeManager.data.*;

class CmdRecipes implements CommandExecutor
{
	@Override
	public boolean onCommand(final CommandSender sender, final Command command, final String label, final String[] args)
	{
		if(args.length <= 0)
		{
			Main.printMessage(sender, Lang.get("command.rmrecipes.usage", "recipes"));
			
			sender.sendMessage(ChatColor.WHITE + "Workbench recipes " + ChatColor.GOLD + Recipes.craft.size() + Recipes.combine.size());
			sender.sendMessage(ChatColor.WHITE + "Furnace recipes " + ChatColor.GOLD + Recipes.smelt.size());
			sender.sendMessage(ChatColor.WHITE + "Furnace fuels: " + ChatColor.GOLD + Recipes.fuels.size());
			
			return true;
		}
		
		final int perPage = 10;
		final String itemStr = args[0].toUpperCase();
		int page = 0;
		Item item;
		boolean ingredient;
		List<Recipe> recipes;
		int recipesNum;
		
		String name = null;
		
		if(sender instanceof Player)
			name = ((Player)sender).getName();
		
		boolean next = itemStr.equals("NEXT");
		boolean prev = itemStr.equals("PREV");
		
		if(next || prev) // next page
		{
			Page data = Recipes.playerPage.get(name);
			
			if(data == null)
				return true;
			
			item = data.item;
			ingredient = data.ingredient;
			recipes = Recipes.getRecipesForItem(item, ingredient);
			recipesNum = recipes.size();
			
			if((prev && (data.page - 1) < 0) || (next && ((data.page + 1) * perPage) > recipesNum))
				return true;
			
			if(next)
				data.page++;
			else
				data.page--;
			
			page = data.page;
			
			Recipes.playerPage.put(name, data);
		}
		else
		{
			if(itemStr.equals("THIS"))
			{
				if(!(sender instanceof Player))
				{
					sender.sendMessage("Only players can use the 'THIS' argument.");
					return true;
				}
				
				final ItemStack itemStack = ((Player)sender).getItemInHand();
				
				if(itemStack == null || itemStack.getTypeId() == 0)
				{
					Main.printMessage(sender, Lang.get("command.rmrecipes.invalidhelditem"));
					return true;
				}
				else
				{
					item = new Item(itemStack);
					item.amount = 0;
					item.enchantments.clear();
				}
			}
			else
			{
				item = Main.processItem(itemStr, 0, false, false, false);
				
				if(item == null)
				{
					Main.printMessage(sender, Lang.get("command.rmrecipes.invaliditem", itemStr.toLowerCase()));
					return true;
				}
			}
			
			ingredient = (args.length > 1);
			recipes = Recipes.getRecipesForItem(item, ingredient);
			recipesNum = recipes.size();
			
			if(recipesNum == 0)
			{
				Main.printMessage(sender, Lang.get((ingredient ? "command.rmrecipes.noingredient" : "command.rmrecipes.noresult"), item.printAuto()));
				return true;
			}
			
			page = 0;
			Recipes.playerPage.put(name, new Page(item, ingredient, page));
		}
		
		final int maxPages = (int)Math.ceil(recipesNum / (0.0 + perPage));
		
		sender.sendMessage(" ");
		Main.printMessage(sender, Lang.get((ingredient ? "command.rmrecipes.listingredient" : "command.rmrecipes.listresult"), item, recipesNum, Lang.get("general.recipe" + (recipesNum == 1 ? "" : "s"))));
		
		if(recipesNum > perPage)
			Main.printMessage(sender, Lang.get("command.rmrecipes.pageofpages", (page + 1), maxPages));
		
		sender.sendMessage(" ");
		
		Recipe r;
		int start = (page * perPage);
		int end = (start + perPage);
		
		for(int i = start; i < end; i++)
		{
			if(i >= recipesNum)
				break;
			
			r = recipes.get(i);
			
			if(r.permission != null && !sender.hasPermission(r.permission))
				continue;
			
			if(sender instanceof Player && !r.usableWorld(((Player)sender).getWorld().getName()))
				continue;
			
			Main.printMessage(sender, r.print());
			sender.sendMessage(" ");
		}
		
		if(recipesNum > end)
		{
			Main.printMessage(sender, Lang.get("command.rmrecipes.nextavailable", (page + 1), maxPages, "recipes next"));
			return true;
		}
		else if(page > 0)
		{
			Main.printMessage(sender, Lang.get("command.rmrecipes.prevavailable", (page + 1), maxPages, "recipes prev"));
		}
		
		sender.sendMessage(" ");
		
		return true;
	}
}

class CmdCheck implements CommandExecutor
{
	@Override
	public boolean onCommand(final CommandSender sender, final Command command, final String label, final String[] args)
	{
		Main.printMessage(sender, Lang.get("command.rmcheck.checking", "recipes"));
		
		if(Main.loadRecipes(true))
			Main.printMessage(sender, Lang.get("command.rmcheck.valid"));
		else if(sender instanceof Player)
			Main.printMessage(sender, Lang.get("command.rmcheck.errors"));
		
		return true;
	}
}

class CmdReload implements CommandExecutor
{
	@Override
	public boolean onCommand(final CommandSender sender, final Command command, final String label, final String[] args)
	{
		Main.printMessage(sender, Lang.get("command.rmreload.reloading"));
		
		Main.loadSettings();
		
		if(Main.loadRecipes(false))
			Main.printMessage(sender, Lang.get("command.rmreload.done"));
		else if(sender instanceof Player)
			Main.printMessage(sender, Lang.get("command.rmreload.errors"));
		
		return true;
	}
}